// *****************************************
// *                                       *
// * MBEDIT.H                              *
// *                                       *
// *                                       *
// * Header file for the                   *
// * MineBombers level editor              *
// * (C) 1998 Yrj Kari-Koskinen           *
// * (C) 1999 Sampo Lankinen               *
// *                                       *
// *****************************************



// * * * * * * * * * * * * * * * * * * * * *
//
// * Global old style #define constants    *
//
// * * * * * * * * * * * * * * * * * * * * *

#define MAX_ENTS    100  // entities alltogether
#define PAL_ENTS     69  // entities on palette
#define LEVEL_SIZE 3060  // level buffer size (45 x 68)
#define HELP_W      390  // help box's width
#define HELP_H      395  // help box's height

// hiirinapit
#define TYHJA 0
#define VASEN 1
#define OIKEA 2
#define KESKI 4

// hiiren sijainti
#define ON_LEVEL    1
#define ON_TOOLBAR  2

#define ON_PAL      4
#define ON_SELECT   8
#define ON_TEXT    16

// tykalut (spritet)
#define KALU_BRUSH    0
#define KALU_PIPETTI  1
#define KALU_FILL     2
#define KALU_SQUARE   3
#define KALU_SELECT   4

// erikoisnppimet
#define K_F1          -59
#define K_F2          -60
#define K_F3          -61
#define K_F4          -62
#define K_F5          -63
#define K_F6          -64
#define K_F7          -65
#define K_F8          -66
#define K_F9          -67
#define K_F10         -68
#define K_F11        -133
#define K_F12        -134

#define K_ALT_Z       -44
#define K_ALT_X       -45

#define K_CTRL_A        1
#define K_CTRL_L       12
#define K_CTRL_N       14
#define K_CTRL_O       15
#define K_CTRL_P       16
#define K_CTRL_Q       17
#define K_CTRL_R       18
#define K_CTRL_S       19
#define K_CTRL_T       20
#define K_CTRL_Z       26
#define K_CTRL_z       28

#define K_CTRL_F4     -97
#define K_CTRL_LEFT  -115
#define K_CTRL_RIGHT -116
#define K_CTRL_UP    -141
#define K_CTRL_DOWN  -145
#define K_CTRL_HOME  -119
#define K_CTRL_END   -117
#define K_CTRL_PGDN  -118
#define K_CTRL_PGUP  -132

#define K_ESC          27

// kysy_nimi-funktion actit
#define SAVE 0
#define LOAD 1


// * * * * * * * * * * * * * * * * * * * * *
//
// * Data structure definitions            *
//
// * * * * * * * * * * * * * * * * * * * * *

struct _entity {  // yks palikka
  char gfx[100];  // 10 x 10
  char name[10], text[45], text2[45];
};

struct _square_tool {
  char begun;
  unsigned char tool;
  int oldx, oldy,
      curx, cury;
};

// **********
// mbedit.cfg
struct _cfg {
  char undo_max, show_logo, tekstit, pos_text, palikka_text, mouse_in_shot,
       level_shot;
};

// * * * * * * * * * * * * * * * * * * * * *
//
// * Functions' prototypes                 *
//
// * * * * * * * * * * * * * * * * * * * * *

// *********
// gifread.c
// *********

// Lukee yhden gif-chunkin, 'GIF_read'-funktiota varten
static int get_next_code(void);

// Lukee tarvittavat tiedot giffist
int GIF_read(char *file, char *buffer, RGB *pal,
             int *width, int *height);


// *********
// screen.cc
// *********

// Piirt pienet metallipalikat ja harmaat 3D-systeemit
void piirra_kehykset(const _entity *ents);
// Piirt 'level':st lytyvill palikoilla taulun nytlle
void piirra_leveli(const _entity *ents, unsigned char *level);
// Piirt 'p':st lytyvt 10x10 pixeli 'bmap':iin kohtaan
void piirra_palikka(BITMAP *bmap, const char *p, const int px, const int py,
                    const int xadd, const int yadd);

// Heitt logon nytlle
void nayta_logo(void);
// Luo helpit 'help':iin
void create_help(BITMAP *help);

// Piirt nytlle viivan bmp:st luetuilla vreill
void undo_line(BITMAP *bmp, int startx, int starty,
               const int endx, const int endy);

// Tytt 'x','y' paikasta lhtien 'tool'-entityll 'level':in alueen
int make_fill(const int x, const int y, const int tool,
              unsigned char *level, const _entity *e);


// ********
// level.cc
// ********

// Lukee 'fn'-tiedostosta palikat 'level':_iin
int lue_leveli(const char *fn, unsigned char *level);
// Tytt 'level':in tunnelilla (0/48)
int tee_tyhja_leveli(unsigned char *level);
// Kirjoittaa 'level':in tiedostoon 'fn', saved kertoo, talletettiinko todella
int kirjota_leveli(char *fn, const unsigned char *level, int saved);


// ********
// mouse.cc
// ********

// Tosi, jos 'maski' pohjassa
int hiirinappi(const int maski);
// Tosi, jos hiiri on 'maski'-alueen pll
int hiiri(const int maski);


// *******
// misc.cc
// *******

// Muuttaa kytettvn tykalun eli hiirikursorin
void set_kalu(const int uusi_kalu, BITMAP **c);

// Kysyy nimen ja kirjoittaa sen ulosteeseen, act on LOAD tai SAVE
void kysy_nimi(char *uloste, const int act);

// Lukee nppimistlt, kunnes joku 'var':in merkeist saadaan ilman casea
int getch_until(const char *var);

// Lukee grafiikat ja tekstit 'e':hen
int lue_graffat(_entity *e);

// Lukee mbedit.cfg-tiedoston ja asettaa cfg_struct:n muuttujat
int lue_cfg(_cfg *c);

// Quittaa
int quit(int &saved, char *levelfn, const unsigned char *level);

// Palauttaa arvotun arvon tietyille entityille (13, 42, 46, 71, 96 ja 2)
unsigned char randomize(const unsigned char entity);
// Sama toisinpin = palauttaa eprandomin arvon :)
unsigned char unrandomize(const unsigned char entity);



// * * * * * * * * * * * * * * * * * * * * *
//
// * Global constants and variables        *
//
// * * * * * * * * * * * * * * * * * * * * *

const char ver[10] = "v0.26";

// hiiren sijainti
char mouse_on = 0, mouse_on_old = 0, tkalu = KALU_BRUSH;

// palauttaa minun entity-taulukkoni koodia vastaavan char:in.
unsigned char ttbl2[256];

// palauttaa char-koodin sijainnin minun entity-taulukossani. (r = randomize)
const unsigned char ttbl[256] = {
  100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
  100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
  100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
  100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
  100, 100, 100, 100, 100, 100, 100, 100,
// minun     levelin
// numba     numba

   0, //  0  48  tunnel
   24, // 1  49  metal
   2, //  2  50  sand      r
   3, //  3  51  sand2
   4, //  4  52  sand3
   5, //  5  53  rocks
   6, //  6  54  rocks2
   7, //  7  55  corner
   8, //  8  56  corner2
   10, // 9  57  corner4
  100, 100, 100, 100, 100, 100, 100,
   9, //  A  65  corner3
   20, // B  66  boulder
   13, // C  67  srock3    r
   14, // D  68  srock4
   15, // E  69  srock5
   16, // F  70  srock6
   42, // G  71  furry     r
   43, // H  72  furry
   44, // I  73  furry
   45, // J  74  furry
   46, // K  75  gmonst    r
   47, // L  76  gmonst2
   48, // M  77  gmonst3
   49, // N  78  gmonst4
   71, // O  79  slime     r
   72, // P  80  slime1
   73, // Q  81  slime2
   74, // R  82  slime3
   96, // S  83  alien     r
   97, // T  84  alien2
   98, // U  85  alien3
   99, // V  86  alien4
   50, // W  87  bomb
   54, // X  88  bbomb
   58, // Y  89  dyna
   89, // Z  90  explsn5
  100, 100, 100, 100, 100, 100,
   90, // a  97  explsn7
  100,
   79, // c  99  rmtB
   80, // d 100  rmtB2
   1, //  e 101  mine
   94, // f 102  blood
   75, // g 103  rmtG
   76, // h 104  rmtG2
   81, // i 105  rmtY
   82, // j 106  rmtY2
   23, // k 107  exit
   22, // l 108  door
   39, // m 109  1staid
  100,
   88, // o 111  bslime
   12, // p 112  srock2
   11, // q 113  srock
  100,
   34, // s 115  diamond
  100, 100, 100,
   51, // w 119  bomb2
   52, // x 120  bomb3
   35, // y 121  grade
  100, 100,
   67, // | 124  napalm2
   53, // } 125  bomb4
   57, // ~ 126  bbomb4
   66, //  127  napalm
   68, //  128  crsfix
   84, //  129  urethn
   77, //  130  rmtR
   78, //  131  rmtR2
   89, //  132  explsn
   89, //  133  explsn2
   90, //  134  explsn6
   91, //  135  explsn0
  100,
   89, //  137  explsn4
   69, //  138  crsfix2
   56, //  139  bbomb3
   55, //  140  bbomb2
   59, //  141  dyna2
   60, //  142  dyna3
   36, //  143  axe1
   37, //  144  axe2
   38, //  145  drill
   25, //  146  brass
   26, //  147  shield
   27, //  148  coins
   28, //  149  egg
   29, //  150  bar
   30, //  151  cross
   31, //  152  sceptre
   32, //  153  rubin
   33, //  154  crown
   85, //  155  urethn2
   40, //  156  tele
   64, //  157  nucl2
   63, //  158  nucl
   65, //  159  nucl3
   87, //  160  plastc2
   86, //  161  plastc
   83, //  162  digger
  100,
   41, //  164  barrel
   70, //  165  grnadeR
   70, //  166  grnadeL
   70, //  167  grnadeD
   70, //  168  grnadeU
  100,
   61, //  170  dyna4
   62, //  171  hopper
   19, //  172  brick3
   18, //  173  brick2
   17, //  174  brick1
   95, //  175  blood2
   89, //  176  explsn3
   90, //  177  explsn8
   91, //  178  explsn9
   93, //  179  1up
   21, //  180  dswitch
  100,
   92, //  182  biti
   100, //       black
   100, 100, 100, 100, 100, 100, // 180
   100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
   100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
   100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
   100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
   100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
   100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
   100, 100, 100, 100, 100, 100};


// The original MineBombers-palette
const char
       pal16[48] = { 0,  0,  0,        // 16 colors
                     63, 63, 63,
                     0,  0,  34,
                     54, 0,  0,
                     0,  40, 0,
                     63, 63, 0,
                     63, 50, 0,
                     53, 39, 0,
                     34, 34, 34,
                     25, 25, 25,
                     17, 17, 17,
                     43, 30, 0,
                     34, 22, 0,
                     26, 16, 0,
                     18, 10, 0,
                     63, 52, 42};

const char toolbar[3][22] = {
   {  0,  2,  5,  6,  7,  8, 20, 21, 22, 23, 50,
     54, 58, 62, 64, 66, 68, 69, 70, 41, 84, 86 },
   {  1, 11, 12, 13,  9, 10, 17, 18, 19, 24, 75,
     76, 77, 78, 79, 80, 81, 82, 83, 88, 85, 87 },
   { 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35,
     36, 37, 38, 39, 40, 71, 42, 96, 46, 94, 95 }};

const char pikkulaatta[25] = {
   9,  8,  8,  8, 8,
  10, 10,  9, 10, 8,
  10,  9,  9,  9, 8,
  10, 10,  9, 10, 8,
  10, 10, 10, 10, 9};
